# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import unittest

from azure.cli.core import azclierror
from azure.cli.core.aaz import exceptions as aazerror


class TestAAZArgShorthandSyntax(unittest.TestCase):

    def test_aaz_shorthand_syntax_simple_value(self):
        from azure.cli.core.aaz._utils import AAZShortHandSyntaxParser
        from azure.cli.core.aaz.exceptions import AAZInvalidShorthandSyntaxError
        parser = AAZShortHandSyntaxParser()

        # test null value
        self.assertEqual(parser("null", is_simple=True), None)
        self.assertEqual(parser("'null'", is_simple=True), 'null')

        self.assertEqual(parser("None", is_simple=True), 'None')
        self.assertEqual(parser("'None'", is_simple=True), 'None')

        # test single quota
        self.assertEqual(parser("''/'", is_simple=True), "'")
        self.assertEqual(parser("'/'", is_simple=True), "/")
        self.assertEqual(parser("'\"'", is_simple=True), '"')

        self.assertEqual(parser('"\'"', is_simple=True), '"\'"')
        self.assertEqual(parser('"\""', is_simple=True), '"\""')
        self.assertEqual(parser("'https://azure.microsoft.com/en-us/'", is_simple=True), "https://azure.microsoft.com/en-us/")
        self.assertEqual(parser("'C:\\Program Files (x86)\\'", is_simple=True), "C:\\Program Files (x86)\\")
        self.assertEqual(parser("'/usr/lib/python3.8/'", is_simple=True), "/usr/lib/python3.8/")

        # test escape character
        self.assertEqual(parser("'\"'", is_simple=True), "\"")
        self.assertEqual(parser("'\'/'", is_simple=True), "\'")

        self.assertEqual(parser("'\b'", is_simple=True), "\b")
        self.assertEqual(parser("'\f'", is_simple=True), "\f")
        self.assertEqual(parser("'\n'", is_simple=True), "\n")
        self.assertEqual(parser("'\r'", is_simple=True), "\r")
        self.assertEqual(parser("'\t'", is_simple=True), "\t")

        self.assertEqual(parser('\b', is_simple=True), '\b')
        self.assertEqual(parser('\f', is_simple=True), '\f')
        self.assertEqual(parser('\n', is_simple=True), '\n')
        self.assertEqual(parser('\r', is_simple=True), '\r')
        self.assertEqual(parser('\t', is_simple=True), '\t')

        self.assertEqual(parser('"\b"', is_simple=True), '"\b"')
        self.assertEqual(parser('"\f"', is_simple=True), '"\f"')
        self.assertEqual(parser('"\n"', is_simple=True), '"\n"')
        self.assertEqual(parser('"\r"', is_simple=True), '"\r"')
        self.assertEqual(parser('"\t"', is_simple=True), '"\t"')

        # whitespace character should be warped in single quotes

        # test normal string
        self.assertEqual(parser("abc", is_simple=True), "abc")
        self.assertEqual(parser("1", is_simple=True), "1")
        self.assertEqual(parser("_", is_simple=True), "_")
        self.assertEqual(parser("''", is_simple=True), '')
        self.assertEqual(parser('""', is_simple=True), '""')
        self.assertEqual(parser("'{'", is_simple=True), "{")
        self.assertEqual(parser('"{"', is_simple=True), '"{"')
        self.assertEqual(parser("'}'", is_simple=True), "}")
        self.assertEqual(parser('"}"', is_simple=True), '"}"')
        self.assertEqual(parser(' ', is_simple=True), ' ')
        self.assertEqual(parser("' '", is_simple=True), ' ')

        self.assertEqual(parser("'{abc'", is_simple=True), "{abc")
        self.assertEqual(parser("'abc}'", is_simple=True), "abc}")
        self.assertEqual(parser("'abc{'", is_simple=True), "abc{")
        self.assertEqual(parser("'}abc'", is_simple=True), "}abc")
        self.assertEqual(parser("'{a:1, b:2}'", is_simple=True), "{a:1, b:2}")

        self.assertEqual(parser("'['", is_simple=True), "[")
        self.assertEqual(parser('"["', is_simple=True), '"["')
        self.assertEqual(parser("']'", is_simple=True), "]")
        self.assertEqual(parser('"]"', is_simple=True), '"]"')

        self.assertEqual(parser("'[abc'", is_simple=True), "[abc")
        self.assertEqual(parser("'abc['", is_simple=True), "abc[")
        self.assertEqual(parser("'abc]'", is_simple=True), "abc]")
        self.assertEqual(parser("']abc'", is_simple=True), "]abc")
        self.assertEqual(parser("'[1, 2, 3]'", is_simple=True), "[1, 2, 3]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("", is_simple=True)

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("'", is_simple=True)

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("'No", is_simple=True)

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("'A'bc", is_simple=True)

    def test_aaz_shorthand_syntax_dict_value(self):
        from azure.cli.core.aaz._utils import AAZShortHandSyntaxParser, AAZBlankArgValue
        from azure.cli.core.aaz.exceptions import AAZInvalidShorthandSyntaxError
        parser = AAZShortHandSyntaxParser()
        self.assertEqual(parser("{}"), {})
        self.assertEqual(parser("{a:1,b:2,c:null,d:''}"), {
            "a": 1,
            "b": 2,
            "c": None,
            "d": ''
        })
        self.assertEqual(parser("{a:'1',b:'2',c:null,d:''}"), {
            "a": "1",
            "b": "2",
            "c": None,
            "d": ''
        })
        self.assertEqual(parser("{a:1,b:2,c:None,d:'',}"), {
            "a": 1,
            "b": 2,
            "c": "None",
            "d": ''
        })
        self.assertEqual(parser("{a:1,b:''/'}"), {
            "a": 1,
            "b": "'"
        })

        self.assertEqual(parser("{a:'1',b:'/'}"), {
            "a": "1",
            "b": "/"
        })

        self.assertEqual(parser("{a:1,b:'//'}"), {
            "a": 1,
            "b": "//"
        })

        self.assertEqual(parser("{a:1,b:/}"), {
            "a": 1,
            "b": "/"
        })

        self.assertEqual(parser("{a:'1',b:'2',c:Null,d:''}"), {
            "a": "1",
            "b": "2",
            "c": "Null",
            "d": ''
        })
        self.assertEqual(parser("{a:1,b:2,c:none,d:'',}"), {
            "a": 1,
            "b": 2,
            "c": "none",
            "d": ''
        })

        self.assertEqual(parser("{a:{a1:' \n '/',a2:2.2}}"), {
            "a": {
                "a1": " \n '",
                "a2": 2.2,
            }
        })

        self.assertEqual(parser("{a:{a1:1,a2:'false'}}"), {
            "a": {
                "a1": 1,
                "a2": 'false',
            }
        })
        self.assertEqual(parser("{a:{a1:'1.1',a2:true},}"), {
            "a": {
                "a1": "1.1",
                "a2": True,
            }
        })
        self.assertEqual(parser("{a:{a1:{},a2:false}}"), {
            "a": {
                "a1": {},
                "a2": False,
            }
        })

        self.assertEqual(parser("{a:{a1,a2:2,c},a3,a4:''}"), {
            "a": {
                "a1": AAZBlankArgValue,
                "a2": 2,
                "c": AAZBlankArgValue,
            },
            "a3": AAZBlankArgValue,
            "a4": '',
        })

        self.assertEqual(parser("{a:1,'',c:1,e}"), {
            "a": 1,
            "": AAZBlankArgValue,
            "c": 1,
            "e": AAZBlankArgValue,
        })

        self.assertEqual(parser("{a:[{prop1:1,prop2:true},{a1:[b,3,3.4,true,false,'true','false',null,c,'d:e \"]'],a2:'2 3\t\"}',a4:'',a3:null,}],e:f,g:null}"), {
            "a": [
                {
                    "prop1": 1,
                    "prop2": True,
                },
                {
                    "a1": ["b", 3, 3.4, True, False, "true", "false", None, "c", 'd:e "]'],
                    "a2": '2 3\t"}',
                    "a4": '',
                    "a3": None,
                }
            ],
            "e": "f",
            "g": None,
        })

        # compare with json
        dt = {}
        s = json.dumps(dt, separators=(',', ':'))
        self.assertEqual(parser(s), dt)

        dt = {"a": 1}
        s = json.dumps(dt, separators=(',', ':'))
        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser(s)

        # other forbidden expression
        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,,c:1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:1]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1b:}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,a:2}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:'/''}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:''/}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,:1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:{}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:1,b:{}}{")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:{a1:1',a2:2}}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:{a1:1'2,a2:2}}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:{a1:1\",a2:2}}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:{a1:1\"2,a2:2}}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:{a1: 1,a2:2}}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a:{a1:1 ,a2:2}}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a'1:1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a\"1:1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a'1:1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{'a\"1':1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{a'1:1}")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("{'a'/1':1}")

    def test_aaz_shorthand_syntax_list_value(self):
        from azure.cli.core.aaz._utils import AAZShortHandSyntaxParser
        from azure.cli.core.aaz.exceptions import AAZInvalidShorthandSyntaxError

        parser = AAZShortHandSyntaxParser()
        self.assertEqual(parser("[]"), [])

        self.assertEqual(parser("[c]"), ["c"])

        self.assertEqual(parser("['c']"), ["c"])

        self.assertEqual(parser("[c,]"), ["c"])

        self.assertEqual(parser("[null]"), [None])
        self.assertEqual(parser("[null,]"), [None])
        self.assertEqual(parser("[None]"), ['None'])
        self.assertEqual(parser("[None,]"), ['None'])

        self.assertEqual(parser("['null',]"), ['null'])

        self.assertEqual(parser("['None',]"), ['None'])

        self.assertEqual(parser("[' ',]"), [' '])

        self.assertEqual(parser("[a,b,c,d]"), [
            "a", "b", "c", "d"
        ])
        self.assertEqual(parser("[a,null,'',d]"), [
            "a", None, "", "d"
        ])
        self.assertEqual(parser("[a,None,'',d,]"), [
            "a", 'None', "", "d"
        ])
        self.assertEqual(parser("[a,[],{},[b,c,d],]"), [
            "a", [], {}, ["b", "c", "d"]
        ])

        # compare with json
        ls = []
        s = json.dumps(ls, separators=(',', ':'))
        self.assertEqual(parser(s), ls)

        ls = [None, {}]
        s = json.dumps(ls, separators=(',', ':'))
        self.assertEqual(parser(s), ls)

        ls = [1, 2, 3, 4]
        s = json.dumps(ls, separators=(',', ':'))
        self.assertEqual(parser(s), ls)

        ls = ["1", "2"]
        s = json.dumps(ls, separators=(',', ':'))
        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser(s)

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[,]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[',]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[ ]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a'b,]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a\"b,]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a b,]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a,b")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a,b,")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a,[b,[c]]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parser("[a,]]")


class TestAAZArg(unittest.TestCase):

    def test_aaz_compound_type_action_key_pattern(self):
        from azure.cli.core.aaz._utils import AAZShortHandSyntaxParser
        key_pattern = AAZShortHandSyntaxParser.partial_value_key_pattern

        test_value = "a.b.c="
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], 'a.b.c')
        self.assertEqual(match[len(match.regs) - 1], '')

        test_value = "a_b-C=aaa"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], 'a_b-C')
        self.assertEqual(match[len(match.regs) - 1], 'aaa')

        test_value = "[10]=aaa"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], '[10]')
        self.assertEqual(match[len(match.regs) - 1], 'aaa')

        test_value = "a_b.C[10]=aaa"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], 'a_b.C[10]')
        self.assertEqual(match[len(match.regs) - 1], 'aaa')

        test_value = "a_b.C[10].D-e_f=aaa"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], 'a_b.C[10].D-e_f')
        self.assertEqual(match[len(match.regs) - 1], 'aaa')

        test_value = "a_b-C={aaa:b}"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], 'a_b-C')
        self.assertEqual(match[len(match.regs) - 1], '{aaa:b}')

        test_value = "a_b-C=[aaa,b]"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], 'a_b-C')
        self.assertEqual(match[len(match.regs) - 1], '[aaa,b]')

        test_value = "a.'/bc sef'/.sss[2]'[1]='/a='"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match[1], "a.'/bc sef'/.sss[2]'[1]")
        self.assertEqual(match[len(match.regs) - 1], "'/a='")

        test_value = "{a_b:aaa}"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match, None)

        test_value = "[aaa,bbb]"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match, None)

        test_value = "a_b.C[abc]=aaa"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match, None)

        test_value = "a_b.C[10].D-e_f"
        match = key_pattern.fullmatch(test_value)
        self.assertEqual(match, None)

    def test_aaz_compound_type_split_key(self):
        from azure.cli.core.aaz._utils import AAZShortHandSyntaxParser, AAZInvalidShorthandSyntaxError
        parse_partial_value_key = AAZShortHandSyntaxParser.parse_partial_value_key
        self.assertEqual(parse_partial_value_key("a.b.c"), ("a", "b", "c"))
        self.assertEqual(parse_partial_value_key("[1]"), (1,))
        self.assertEqual(parse_partial_value_key("[-10]"), (-10,))
        self.assertEqual(parse_partial_value_key("a1[5].b[2].c"), ("a1", 5, "b", 2, "c"))
        self.assertEqual(parse_partial_value_key("a.'/bc sef'/.sss[2]'[1]"), ("a", "/bc sef'.sss[2]", 1))
        self.assertEqual(parse_partial_value_key("'/bc sef'/.sss[2]'"), ("/bc sef'.sss[2]", ))
        self.assertEqual(parse_partial_value_key("'/bc sef'/.sss[2]'.c"), ("/bc sef'.sss[2]", "c"))

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parse_partial_value_key(".a")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parse_partial_value_key(".[5]")

        with self.assertRaises(AAZInvalidShorthandSyntaxError):
            parse_partial_value_key("b.a.'/bc sef'/.sss[2]")

    def test_aaz_str_arg(self):
        from azure.cli.core.aaz._arg import AAZStrArg, AAZArgumentsSchema, AAZListArg
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.work_day = AAZStrArg(
            options=["--work-day", "-d"],
            enum={
                "1": "Monday",
                "2": "Tuesday",
                "3": "Wednesday",
                "4": "Thursday",
                "5": "Friday",
                "6": "Saturday",
                "7": "Sunday",
                "Mon": "Monday",
                "Tue": "Tuesday",
                "Wed": "Wednesday",
                "Thu": "Thursday",
                "Fri": "Friday",
                "Sat": "Saturday",
                "Sun": "Sunday",
            },
            nullable=True,
            blank="Sunday"
        )
        schema.tasks = AAZListArg(
            options=["--tasks"],
            singular_options=["--task", "-t"],
        )
        schema.tasks.Element = AAZStrArg(
            enum={"task1", "task2"},
            enum_support_extension=True,
        )
        self.assertFalse(has_value(v.work_day))

        arg = schema.work_day.to_cmd_arg("work_day")
        self.assertEqual(len(arg.choices), 14)
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, "1")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, "Monday")

        action.setup_operations(dest_ops, "2")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, "Tuesday")

        action.setup_operations(dest_ops, "Thu")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, "Thursday")

        action.setup_operations(dest_ops, "fri")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, "Friday")

        # null value
        action.setup_operations(dest_ops, 'null')
        self.assertEqual(len(dest_ops._ops), 5)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, None)  # must use '== None', because 'is None' will not work

        # blank value
        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 6)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, "Sunday")

        # null value
        action.setup_operations(dest_ops, 'null')
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, None)

        # test invalid operations
        with self.assertRaises(azclierror.InvalidArgumentValueError):
            action.setup_operations(dest_ops, '1234')
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "work_day")
        self.assertEqual(v.work_day, None)

        # Task argument
        arg = schema.tasks.to_cmd_arg("tasks")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, ["[0,'null',false,'true',1.2,'1.2','10']"])
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "tasks")
        self.assertEqual(v.tasks.to_serialized_data(), ["0", "null", "false", "true", "1.2", "1.2", "10"])

        # New argument
        schema.name = AAZStrArg(options=["--name", "-n"])
        arg = schema.name.to_cmd_arg("work_day")
        action = arg.type.settings["action"]
        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, "test name")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "name")
        self.assertEqual(v.name, "test name")

        action.setup_operations(dest_ops, "")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "name")
        self.assertEqual(v.name, "")

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "null")
        with self.assertRaises(aazerror.AAZInvalidShorthandSyntaxError):
            action.setup_operations(dest_ops, "'l")
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, None)

        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "name")
        self.assertEqual(v.name, "")

        action.setup_operations(dest_ops, " aa' l_;{]'")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "name")
        self.assertEqual(v.name, " aa' l_;{]'")

    def test_aaz_int_arg(self):
        from azure.cli.core.aaz._arg import AAZIntArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.score = AAZIntArg(
            options=["--score", "-s"],
            enum={
                "A": 100,
                "B": 90,
                "C": 80,
                "D": 0,
            },
            enum_support_extension=True,
            nullable=True,
            blank=0
        )

        self.assertFalse(has_value(v.score))

        arg = schema.score.to_cmd_arg("score")
        self.assertEqual(arg.choices, None)
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, "A")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, 100)

        # null value
        action.setup_operations(dest_ops, "null")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, None)

        # blank value
        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, 0)

        # null value
        action.setup_operations(dest_ops, "null")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, None)

         # extension value
        action.setup_operations(dest_ops, "1234")
        self.assertEqual(len(dest_ops._ops), 5)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, 1234)

        # extension invalid value
        with self.assertRaises(azclierror.InvalidArgumentValueError):
            action.setup_operations(dest_ops, "12A34")

        # credit argument
        schema.credit = AAZIntArg(options=["--credit", "-c"])
        arg = schema.credit.to_cmd_arg("credit")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, "-100")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, -100)

        action.setup_operations(dest_ops, "0")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, 0)

        action.setup_operations(dest_ops, "100")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, 100)

        action.setup_operations(dest_ops, "'10'")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, 10)

        # test blank
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, None)

        # test null
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "null")

        with self.assertRaises(ValueError):
            action.setup_operations(dest_ops, "100.123")

        with self.assertRaises(ValueError):
            action.setup_operations(dest_ops, " ")

    def test_aaz_float_arg(self):
        from azure.cli.core.aaz._arg import AAZFloatArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.score = AAZFloatArg(
            options=["--score", "-s"],
            enum={
                "A": 100.0,
                "B": 90.0,
                "C": 80.0,
                "D": 0.0,
            },
            enum_support_extension=True,
            nullable=True,
            blank=0.0
        )
        self.assertFalse(has_value(v.score))

        arg = schema.score.to_cmd_arg("score")
        self.assertEqual(arg.choices, None)
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, "A")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, 100.0)

        # null value
        action.setup_operations(dest_ops, "null")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, None)

        # blank value
        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, 0.0)

        # null value
        action.setup_operations(dest_ops, "null")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "score")
        self.assertEqual(v.score, None)
        
        # extension value
        from unittest import mock
        with mock.patch('azure.cli.core.aaz._arg.logger') as mock_logger:
            action.setup_operations(dest_ops, "1234")
            self.assertEqual(len(dest_ops._ops), 5)
            dest_ops.apply(v, "score")
            self.assertEqual(v.score, 1234.0)
            call_args = mock_logger.warning.call_args
            self.assertEqual("Use extended value '%s' outside choices %s.", call_args[0][0])
            self.assertEqual("1234.0", call_args[0][1])

        # extension invalid value
        with self.assertRaises(azclierror.InvalidArgumentValueError):
            action.setup_operations(dest_ops, "12A34")

        # credit argument
        schema.credit = AAZFloatArg(options=["--credit", "-c"])
        arg = schema.credit.to_cmd_arg("credit")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, "-100")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, -100.0)

        action.setup_operations(dest_ops, "0.23")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, 0.23)

        action.setup_operations(dest_ops, "100.1")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, 100.1)

        action.setup_operations(dest_ops, "'10.123'")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "credit")
        self.assertEqual(v.credit, 10.123)

        # test blank
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, None)

        # test null
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "null")

        with self.assertRaises(ValueError):
            action.setup_operations(dest_ops, " ")

    def test_aaz_bool_arg(self):
        from azure.cli.core.aaz._arg import AAZBoolArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.enable = AAZBoolArg(options=["--enable", "-e"])
        self.assertFalse(has_value(v.enable))

        arg = schema.enable.to_cmd_arg("enable")
        self.assertEqual(len(arg.choices), 10)
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, True)

        action.setup_operations(dest_ops, "false")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, False)

        action.setup_operations(dest_ops, "true")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, True)

        action.setup_operations(dest_ops, "f")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, False)

        action.setup_operations(dest_ops, "t")
        self.assertEqual(len(dest_ops._ops), 5)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, True)

        action.setup_operations(dest_ops, "no")
        self.assertEqual(len(dest_ops._ops), 6)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, False)

        action.setup_operations(dest_ops, "yes")
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, True)

        action.setup_operations(dest_ops, "n")
        self.assertEqual(len(dest_ops._ops), 8)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, False)

        action.setup_operations(dest_ops, "y")
        self.assertEqual(len(dest_ops._ops), 9)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, True)

        action.setup_operations(dest_ops, "0")
        self.assertEqual(len(dest_ops._ops), 10)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, False)

        action.setup_operations(dest_ops, "1")
        self.assertEqual(len(dest_ops._ops), 11)
        dest_ops.apply(v, "enable")
        self.assertEqual(v.enable, True)

        # null value
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "null")

        # null value
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "null")

        schema.started = AAZBoolArg(
            options=["--started", "-s"],
            nullable=True,
            blank=False,
        )
        arg = schema.started.to_cmd_arg("started")
        self.assertEqual(len(arg.choices), 10)
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        # null value
        action.setup_operations(dest_ops, "null")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "started")
        self.assertEqual(v.started, None)

        # blank value
        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "started")
        self.assertEqual(v.started, False)

        action.setup_operations(dest_ops, "TRUE")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "started")
        self.assertEqual(v.started, True)

        # self.assertEqual(len(dest_ops._ops), 12)
        # dest_ops.apply(v, "enable")
        # self.assertEqual(v.enable, False)
        #
        # action.setup_operations(dest_ops, "true")
        # self.assertEqual(len(dest_ops._ops), 3)
        # dest_ops.apply(v, "enable")
        # self.assertEqual(v.enable, True)

    def test_aaz_list_arg(self):
        from azure.cli.core.aaz._arg import AAZListArg, AAZStrArg, AAZArgumentsSchema, AAZObjectArg
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations, _ELEMENT_APPEND_KEY
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.names = AAZListArg(
            options=["--names", "--ns"],
            singular_options=["--name", "-n"]
        )
        schema.names.Element = AAZStrArg(
            nullable=True,
            blank="a blank value"
        )
        schema.user_assigned = AAZListArg(
            options=["--user-assigned"],
            blank=["1", "2"],
        )
        schema.user_assigned.Element = AAZStrArg(
        )
        schema.objs = AAZListArg(
            options=["--objs"],
            singular_options=["--obj"]
        )
        element = schema.objs.Element = AAZObjectArg()
        element.attr = AAZStrArg(options=["--attr"])
        element.prop = AAZStrArg(options=["--prop"])

        self.assertFalse(has_value(v.names))

        arg = schema.names.to_cmd_arg("names")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        # null value
        action.setup_operations(dest_ops, ["null"])
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, [None, ])

        action.setup_operations(dest_ops, ["[a,b,'c',' ']"])
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ['a', 'b', 'c', ' '])

        action.setup_operations(dest_ops, ["[2]=efg", "[-1]='null'", "[0]="])
        self.assertEqual(len(dest_ops._ops), 5)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ['a blank value', 'b', 'efg', 'null'])

        action.setup_operations(dest_ops, ["c", "d"])
        self.assertEqual(len(dest_ops._ops), 6)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ['c', 'd'])

        action.setup_operations(dest_ops, ["[]"])
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, [])

        action.setup_operations(dest_ops, ["a"])
        self.assertEqual(len(dest_ops._ops), 8)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["a"])

        action.setup_operations(dest_ops, ["", "''"])
        self.assertEqual(len(dest_ops._ops), 9)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["", ""])

        action.setup_operations(dest_ops, ["a", 'null', 'None', "b", ""])
        self.assertEqual(len(dest_ops._ops), 10)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["a", None, 'None', "b", ""])

        # blank value
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, None)

        action.setup_operations(dest_ops, ["[]"])
        self.assertEqual(len(dest_ops._ops), 11)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, [])

        # test singular action
        singular_action = schema.names.Element._build_cmd_action()

        singular_action.setup_operations(dest_ops, ["a"], prefix_keys=[_ELEMENT_APPEND_KEY])
        self.assertEqual(len(dest_ops._ops), 12)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["a"])

        singular_action.setup_operations(dest_ops, ["b"], prefix_keys=[_ELEMENT_APPEND_KEY])
        self.assertEqual(len(dest_ops._ops), 13)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["a", "b"])

        singular_action.setup_operations(dest_ops, None, prefix_keys=[_ELEMENT_APPEND_KEY])
        self.assertEqual(len(dest_ops._ops), 14)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["a", "b", "a blank value"])

        singular_action.setup_operations(dest_ops, [""], prefix_keys=[_ELEMENT_APPEND_KEY])
        self.assertEqual(len(dest_ops._ops), 15)
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ["a", "b", "a blank value", ""])

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)
        singular_action = schema.objs.Element._build_cmd_action()
        singular_action.setup_operations(dest_ops, ["attr=a"], prefix_keys=[_ELEMENT_APPEND_KEY])
        singular_action.setup_operations(dest_ops, ["prop=b"], prefix_keys=[-1])
        dest_ops.apply(v, "objs")
        self.assertEqual(v.objs, [{"attr": "a", "prop": "b"}])

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)
        action = schema.user_assigned._build_cmd_action()
        action.setup_operations(dest_ops, [])
        dest_ops.apply(v, "user_assigned")
        self.assertEqual(v.user_assigned, ["1", "2"])

        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "user_assigned")
        self.assertEqual(v.user_assigned, ["1", "2"])

    def test_aaz_dict_arg(self):
        from azure.cli.core.aaz._arg import AAZDictArg, AAZStrArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.tags = AAZDictArg(
            options=["--tags", "-t"],
            blank={"ab": '1', "bc": '2'},
        )
        schema.tags.Element = AAZStrArg(
            nullable=True,
            blank="a blank value"
        )

        self.assertFalse(has_value(v.tags))

        arg = schema.tags.to_cmd_arg("tags")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        # null value
        action.setup_operations(dest_ops, ["a=null", "b=None"])
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"a": None, "b": 'None'})

        action.setup_operations(dest_ops, ["b=6", "c="])
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"a": None, "b": "6", "c": "a blank value"})

        action.setup_operations(dest_ops, ["{ab:1,bc:2,cd:'null'}"])
        self.assertEqual(len(dest_ops._ops), 5)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"ab": '1', "bc": '2', 'cd': 'null'})

        action.setup_operations(dest_ops, ["{}"])
        self.assertEqual(len(dest_ops._ops), 6)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {})

        # blank value
        action.setup_operations(dest_ops, [])
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"ab": '1', "bc": '2'})

        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 8)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"ab": '1', "bc": '2'})

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["=1"])

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["null"])

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["null"])

        with self.assertRaises(aazerror.AAZInvalidShorthandSyntaxError):
            action.setup_operations(dest_ops, ["{c:}"])

    def test_aaz_freeform_dict_arg(self):
        from azure.cli.core.aaz._arg import AAZFreeFormDictArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.tags = AAZFreeFormDictArg(
            options=["--tags", "-t"],
            nullable=True,
            blank={"blank": True}
        )

        self.assertFalse(has_value(v.tags))

        arg = schema.tags.to_cmd_arg("tags")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        # null value
        action.setup_operations(dest_ops, ["null"])
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, None)

        # empty dict
        action.setup_operations(dest_ops, ["{}"])
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {})

        # partial value
        action.setup_operations(dest_ops, ["a=1", "b=false", "c=abc", "d='1'", "e='null'"])
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"a": 1, "b": False, "c": "abc", "d": "1", "e": 'null'})

        # json string
        action.setup_operations(dest_ops, ['{"a": 1, "c": false, "d": "string", "e": [1, "str", null]}'])
        self.assertEqual(len(dest_ops._ops), 8)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"a": 1, "c": False, "d": "string", "e": [1, "str", None]})

        # blank value
        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 9)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"blank": True})

        # with shorthand syntax
        action.setup_operations(dest_ops, ["{a:1,b:false,c:abc,d:'1',e:'null'}"])
        self.assertEqual(len(dest_ops._ops), 10)
        dest_ops.apply(v, "tags")
        self.assertEqual(v.tags, {"a": 1, "b": False, "c": "abc", "d": "1", "e": 'null'})

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["'null'"])

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["'123'"])

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["123"])

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, ["[1, 2, 3]"])
    
    def test_aaz_any_arg(self):
        # verify the shell_safe_json_parse function
        from azure.cli.core.aaz._arg import AAZAnyTypeArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        from azure.cli.core.util import shell_safe_json_parse
        schema = AAZArgumentsSchema()
        v = schema()

        schema.any = AAZAnyTypeArg(
            options=["--any", "-a"],
            nullable=True,
            blank={"blank": True}
        )

        self.assertFalse(has_value(v.any))

        arg = schema.any.to_cmd_arg("any")
        action = arg.type.settings["action"]
        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        # null value
        action.setup_operations(dest_ops, "null")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, None)

        # empty dict
        action.setup_operations(dest_ops, "{}")
        self.assertEqual(len(dest_ops._ops), 2)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, {})

        # partial value should raise error
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "a=1")
        # assign a partial value like string
        action.setup_operations(dest_ops, "'a=1'")
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, "a=1")

        # empty list
        action.setup_operations(dest_ops, "[]")
        self.assertEqual(len(dest_ops._ops), 4)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, [])

        # blank value
        action.setup_operations(dest_ops, None)
        self.assertEqual(len(dest_ops._ops), 5)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, {"blank": True})

        # json string
        action.setup_operations(dest_ops, '{"a": 1, "c": false, "d": "string", "e": [1, "str", null]}')
        self.assertEqual(len(dest_ops._ops), 6)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, {"a": 1, "c": False, "d": "string", "e": [1, "str", None]})

        # with shorthand syntax
        action.setup_operations(dest_ops, "{a:1,b:false,c:abc,d:'1',e:'null'}")
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, {"a": 1, "b": False, "c": "abc", "d": "1", "e": 'null'})

        # test strings
        action.setup_operations(dest_ops, "'null'")
        self.assertEqual(len(dest_ops._ops), 8)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, 'null')

        action.setup_operations(dest_ops, "'1'")
        self.assertEqual(len(dest_ops._ops), 9)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, '1')

        action.setup_operations(dest_ops, "'false'")
        self.assertEqual(len(dest_ops._ops), 10)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, 'false')

        action.setup_operations(dest_ops, "'true'")
        self.assertEqual(len(dest_ops._ops), 11)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, 'true')

        # test int
        action.setup_operations(dest_ops, "123")
        self.assertEqual(len(dest_ops._ops), 12)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, 123)

        # test float
        action.setup_operations(dest_ops, "123.456")
        self.assertEqual(len(dest_ops._ops), 13)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, 123.456)

        # test bool
        action.setup_operations(dest_ops, "true")
        self.assertEqual(len(dest_ops._ops), 14)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, True)

        action.setup_operations(dest_ops, "false")
        self.assertEqual(len(dest_ops._ops), 15)
        dest_ops.apply(v, "any")
        self.assertEqual(v.any, False)

        # test without nullable and blank value
        schema.any2 = AAZAnyTypeArg(
            options=["--any2"],
        )
        self.assertFalse(has_value(v.any2))

        arg = schema.any2.to_cmd_arg("any")
        action = arg.type.settings["action"]
        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)
        
        action.setup_operations(dest_ops, "[]")
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "any2")
        self.assertEqual(v.any2, [])

        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, None)
        
        with self.assertRaises(aazerror.AAZInvalidValueError):
            action.setup_operations(dest_ops, "null")

        self.assertEqual(shell_safe_json_parse("true"), True)
        self.assertEqual(shell_safe_json_parse("false"), False)
        self.assertEqual(shell_safe_json_parse("'true'"), "true")
        self.assertEqual(shell_safe_json_parse('"false"'), "false")
        self.assertEqual(shell_safe_json_parse("null"), None)
        self.assertEqual(shell_safe_json_parse("'null'"), "null")
        self.assertEqual(shell_safe_json_parse('"null"'), "null")
        self.assertEqual(shell_safe_json_parse("123"), 123)
        self.assertEqual(shell_safe_json_parse("'123'"), "123")
        self.assertEqual(shell_safe_json_parse('"123"'), "123")
        self.assertEqual(shell_safe_json_parse("[]"), [])
        self.assertEqual(shell_safe_json_parse("['a', 'b', 'c']"), ["a", "b", "c"])
        self.assertEqual(shell_safe_json_parse("[1, 2, 3]"), [1, 2, 3])
        self.assertEqual(shell_safe_json_parse("{}"), {})
        self.assertEqual(shell_safe_json_parse('{"a": 1, "b": 2, "c": 3}'), {"a": 1, "b": 2, "c": 3})
        self.assertEqual(shell_safe_json_parse(r"'{\"a\":1,\"b\":2,\"c\":3}'"), '{"a":1,"b":2,"c":3}')

    def test_aaz_object_arg(self):
        from azure.cli.core.aaz._arg import AAZDictArg, AAZListArg, AAZObjectArg, AAZIntArg, AAZBoolArg, AAZFloatArg, \
            AAZStrArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        schema = AAZArgumentsSchema()
        v = schema()

        schema.properties = AAZObjectArg(
            options=["--prop", "-p"],
            nullable=True
        )

        schema.properties.enable = AAZBoolArg(
            options=["enable"],
            nullable=True,
        )
        schema.properties.tags = AAZDictArg(
            options=["tags"],
            nullable=True
        )
        schema.properties.identities = AAZDictArg(
            options=["identities"],
        )
        schema.properties.identities.Element = AAZObjectArg(
            blank={},
            nullable=True,
        )
        schema.properties.tags.Element = AAZIntArg(
            blank="0"
        )
        schema.properties.vnets = AAZListArg(
            options=["vnets"],
            singular_options=["vnet"],
            nullable=True
        )
        schema.properties.vnets.Element = AAZObjectArg()
        schema.properties.vnets.Element.id = AAZStrArg(
            options=["id"],
            blank="666"
        )

        schema.properties.new_i_pv6 = AAZStrArg(
            options=["new_ipv6"],
        )

        schema.properties.pt = AAZFloatArg(
            options=["pt"],
            nullable=True,
            blank="0.1"
        )

        self.assertFalse(has_value(v.properties.identities))
        self.assertFalse(has_value(v.properties.vnets))
        self.assertFalse(has_value(v.properties.pt))
        self.assertFalse(has_value(v.properties))

        arg = schema.properties.to_cmd_arg("properties")
        action = arg.type.settings["action"]

        dest_ops = AAZArgActionOperations()
        self.assertEqual(len(dest_ops._ops), 0)

        action.setup_operations(dest_ops, ["{enable:false,tags:{a:1,3:2},vnets:[{id:/123}],pt:12.123}"])
        self.assertEqual(len(dest_ops._ops), 1)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {
            "enable": False,
            "tags": {
                "a": 1,
                "3": 2,
            },
            "vnets": [
                {"id": "/123"},
            ],
            "pt": 12.123
        })

        action.setup_operations(dest_ops, ["vnet.id=223", "vnet={id:456}"])
        self.assertEqual(len(dest_ops._ops), 3)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties.to_serialized_data(), {
            "enable": False,
            "tags": {
                "a": 1,
                "3": 2,
            },
            "vnets": [
                {"id": "/123"},
                {"id": "223"},
                {"id": "456"}
            ],
            "pt": 12.123
        })

        action.setup_operations(dest_ops, ["pt=", "enable=null", "vnets=[]"])
        self.assertEqual(len(dest_ops._ops), 6)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {
            "enable": None,
            "tags": {
                "a": 1,
                "3": 2,
            },
            "vnets": [],
            "pt": 0.1
        })

        action.setup_operations(dest_ops, ["{enable:false,pt,tags:{a:1,3:2,c},vnets:[{id}],identities:{a:{},'http://b/c/d/e'}}"])
        self.assertEqual(len(dest_ops._ops), 7)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {
            "enable": False,
            "tags": {
                "a": 1,
                "3": 2,
                "c": 0,
            },
            "vnets": [
                {"id": "666"},
            ],
            "identities": {
                "a": {},
                "http://b/c/d/e": {},
            },
            "pt": 0.1
        })

        action.setup_operations(dest_ops, ["identities.'http://b.p['/]/c'=", "identities.a=null"])
        self.assertEqual(len(dest_ops._ops), 9)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {
            "enable": False,
            "tags": {
                "a": 1,
                "3": 2,
                "c": 0,
            },
            "vnets": [
                {"id": "666"},
            ],
            "identities": {
                "a": None,
                "http://b/c/d/e": {},
                "http://b.p[']/c": {},
            },
            "pt": 0.1
        })

        action.setup_operations(dest_ops, ["{}"])
        self.assertEqual(len(dest_ops._ops), 10)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {})

        action.setup_operations(dest_ops, ["null"])
        self.assertEqual(len(dest_ops._ops), 11)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, None)

        action.setup_operations(dest_ops, ["{enable:True,tags:null,vnets:null,pt:12.123,newIPv6:'00:00:00'}"])
        self.assertEqual(len(dest_ops._ops), 12)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {
            "enable": True,
            "tags": None,
            "vnets": None,
            "pt": 12.123,
            "new_i_pv6": '00:00:00'
        })

        # test assign the int value to the float field, it should be converted to float
        action.setup_operations(dest_ops, ["{enable:True,tags:null,vnets:null,pt:12,newIPv6:'00:00:00'}"])
        self.assertEqual(len(dest_ops._ops), 13)
        dest_ops.apply(v, "properties")
        self.assertEqual(v.properties, {
            "enable": True,
            "tags": None,
            "vnets": None,
            "pt": 12.0,
            "new_i_pv6": '00:00:00'
        })


    def test_aaz_has_value_for_buildin(self):
        from azure.cli.core.aaz import has_value, AAZUndefined
        self.assertTrue(has_value(0))
        self.assertTrue(has_value(""))
        self.assertTrue(has_value(False))
        self.assertTrue(has_value(None))

        self.assertFalse(has_value(AAZUndefined))

    def test_aaz_registered_arg(self):
        from azure.cli.core.aaz._arg import AAZStrArg, AAZObjectArg, AAZBoolArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value
        from azure.cli.core.aaz.exceptions import AAZUnregisteredArg
        schema = AAZArgumentsSchema()
        v = schema()

        schema.work_day = AAZStrArg(
            options=["--work-day", "-d"],
            enum={
                "1": "Monday",
                "2": "Tuesday",
                "3": "Wednesday",
                "4": "Thursday",
                "5": "Friday",
                "6": "Saturday",
                "7": "Sunday",
                "Mon": "Monday",
                "Tue": "Tuesday",
                "Wed": "Wednesday",
                "Thu": "Thursday",
                "Fri": "Friday",
                "Sat": "Saturday",
                "Sun": "Sunday",
            },
            nullable=True,
            blank="Sunday"
        )
        self.assertTrue(schema.work_day._registered)

        arg = schema.work_day.to_cmd_arg("work_day")

        schema.work_day._registered = False
        with self.assertRaises(AAZUnregisteredArg):
            schema.work_day.to_cmd_arg("work_day")

        schema.properties = AAZObjectArg(
            options=["--prop", "-p"],
            nullable=True
        )

        schema.properties.enable = AAZBoolArg(
            options=["enable"],
            nullable=True,
        )

        self.assertTrue(schema.properties._registered)
        arg = schema.properties.to_cmd_arg("properties")

        schema.properties._registered = False
        with self.assertRaises(AAZUnregisteredArg):
            schema.properties.to_cmd_arg("properties")

    def test_aaz_configured_default_arg(self):
        from azure.cli.core.aaz._arg import AAZResourceGroupNameArg, AAZResourceLocationArg, AAZStrArg, AAZIntArg,\
            AAZArgumentsSchema
        schema = AAZArgumentsSchema()
        v = schema()

        schema.resource_group = AAZResourceGroupNameArg()
        schema.location = AAZResourceLocationArg()
        schema.name = AAZStrArg(configured_default="specialname")
        schema.count = AAZIntArg(configured_default="specialcount")
        arg = schema.resource_group.to_cmd_arg("resource_group")
        self.assertEqual(arg.type.settings['configured_default'], 'group')
        arg = schema.location.to_cmd_arg("location")
        self.assertEqual(arg.type.settings['configured_default'], 'location')
        arg = schema.name.to_cmd_arg("name")
        self.assertEqual(arg.type.settings['configured_default'], 'specialname')
        arg = schema.count.to_cmd_arg("count")
        self.assertEqual(arg.type.settings['configured_default'], 'specialcount')

    def test_aaz_generic_update_arg(self):
        from azure.cli.core.aaz._arg import (AAZGenericUpdateAddArg, AAZGenericUpdateSetArg, AAZGenericUpdateRemoveArg,
                                             AAZArgumentsSchema)
        schema = AAZArgumentsSchema()
        schema.generic_update_add = AAZGenericUpdateAddArg()
        schema.generic_update_set = AAZGenericUpdateSetArg()
        schema.generic_update_remove = AAZGenericUpdateRemoveArg()
        arg = schema.generic_update_add.to_cmd_arg("add")
        self.assertEqual(arg.type.settings['help'], 'Add an object to a list of objects by specifying a path and key value pairs.  Example: `--add property.listProperty <key=value, string or JSON string>`')
        arg = schema.generic_update_set.to_cmd_arg("set")
        self.assertEqual(arg.type.settings['help'], 'Update an object by specifying a property path and value to set.  Example: `--set property1.property2=<value>`')
        arg = schema.generic_update_remove.to_cmd_arg("remove")
        self.assertEqual(arg.type.settings['help'], 'Remove a property or an element from a list.  Example: `--remove property.list <indexToRemove>` OR `--remove propertyToRemove`')


class TestAAZArgUtils(unittest.TestCase):

    def test_assign_aaz_list_arg(self):
        from azure.cli.core.aaz.utils import assign_aaz_list_arg
        from azure.cli.core.aaz._arg import AAZListArg, AAZStrArg, AAZObjectArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value, AAZUndefined
        schema = AAZArgumentsSchema()
        v = schema()

        schema.names = AAZListArg(
            options=["--names", "--ns"],
            singular_options=["--name", "-n"],
            nullable=True
        )
        schema.names.Element = AAZStrArg(
            nullable=True,
            blank="a blank value"
        )
        schema.items = AAZListArg(
            nullable=True,
        )
        schema.items.Element = AAZObjectArg(
            nullable=True,
        )
        schema.items.Element.name = AAZStrArg()

        # AAZUndefined
        self.assertFalse(has_value(v.names))
        v.items = assign_aaz_list_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertFalse(has_value(v.items))

        v.items = [{"name": "1"}]
        v.items = assign_aaz_list_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, [{"name": "1"}])

        arg = schema.names.to_cmd_arg("names")
        action = arg.type.settings["action"]

        # null value
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["null"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, None)

        v.items = assign_aaz_list_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, None)

        # null element value
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["[0]=null"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, [None, ])

        v.items = assign_aaz_list_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, [None, ])

        # element
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["1", "2", "3"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, ['1', '2', '3'])
        self.assertFalse(v.names._is_patch)

        v.items = assign_aaz_list_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, [{"name": '1'}, {"name": '2'}, {"name": "3"}])
        self.assertFalse(v.items._is_patch)

        # patch element
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["[3]=n5"])
        dest_ops.apply(v, "names")
        self.assertTrue(v.names._is_patch)
        self.assertEqual(len(v.names), 4)
        self.assertFalse(has_value(v.items[0]))
        self.assertFalse(has_value(v.items[1]))
        self.assertFalse(has_value(v.items[2]))
        self.assertEqual(v.names[3], 'n5')
        v.items = assign_aaz_list_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertTrue(v.items._is_patch)
        self.assertEqual(len(v.items), 4)
        self.assertFalse(has_value(v.items[0]))
        self.assertFalse(has_value(v.items[1]))
        self.assertFalse(has_value(v.items[2]))
        self.assertEqual(v.items[3], {"name": 'n5'})

    def test_assign_aaz_dict_arg(self):
        from azure.cli.core.aaz.utils import assign_aaz_dict_arg
        from azure.cli.core.aaz._arg import AAZDictArg, AAZStrArg, AAZObjectArg, AAZArgumentsSchema
        from azure.cli.core.aaz._arg_action import AAZArgActionOperations
        from azure.cli.core.aaz import has_value, AAZUndefined
        schema = AAZArgumentsSchema()
        v = schema()

        schema.names = AAZDictArg(
            options=["--names", "--ns"],
            nullable=True
        )
        schema.names.Element = AAZStrArg(
            nullable=True,
            blank="a blank value"
        )
        schema.items = AAZDictArg(
            nullable=True,
        )
        schema.items.Element = AAZObjectArg(
            nullable=True,
        )
        schema.items.Element.name = AAZStrArg()

        # AAZUndefined
        self.assertFalse(has_value(v.names))
        v.items = assign_aaz_dict_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertFalse(has_value(v.items))

        v.items = {"a": {"name": "1"}}
        v.items = assign_aaz_dict_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, {"a": {"name": "1"}})

        arg = schema.names.to_cmd_arg("names")
        action = arg.type.settings["action"]

        # null value
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["null"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, None)

        v.items = assign_aaz_dict_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, None)

        # null element value
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["a=null"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, {'a': None})

        v.items = assign_aaz_dict_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertEqual(v.items, {'a': None})

        # element
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["{a:a1,b:b2}"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, {'a': 'a1', 'b': 'b2'})
        self.assertFalse(v.names._is_patch)

        v.items = assign_aaz_dict_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertFalse(v.items._is_patch)
        self.assertEqual(v.items, {'a': {"name": 'a1'}, 'b': {"name": 'b2'}})

        # element with patch
        v = schema()
        dest_ops = AAZArgActionOperations()

        action.setup_operations(dest_ops, ["a=1", "b=2", "c=3"])
        dest_ops.apply(v, "names")
        self.assertEqual(v.names, {'a': '1', 'b': '2', 'c': '3'})
        self.assertTrue(v.names._is_patch)

        v.items = assign_aaz_dict_arg(v.items, v.names, element_transformer=lambda _, name: {"name": name})
        self.assertTrue(v.items._is_patch)
        self.assertEqual(v.items, {'a': {"name": '1'}, 'b': {"name": '2'}, 'c': {"name": "3"}})


class TestAazArgCompleter(unittest.TestCase):
    from azure.cli.core.decorators import Completer

    @staticmethod
    @Completer
    def custom_completer(cmd, prefix, namespace):
        return ['a', 'b']

    def test_custom_completer(self):
        from azure.cli.core.aaz._arg import AAZStrArg, AAZArgumentsSchema, AAZUnregisteredArg

        schema = AAZArgumentsSchema()
        v = schema()

        schema.foo = AAZStrArg(
            options=['foo'],
            nullable=True,
            completer=self.custom_completer
        )
        args = {}
        for name, field in schema._fields.items():
            # generate command arguments from argument schema.
            try:
                args[name] = field.to_cmd_arg(name)
            except AAZUnregisteredArg:
                continue
        self.assertEqual(args['foo'].completer, schema.foo._completer)
        self.assertEqual(args['foo'].completer, self.custom_completer)

    def test_completer_override(self):
        from azure.cli.core.aaz._arg import (AAZResourceLocationArg, AAZArgumentsSchema, AAZUnregisteredArg,
                                             AAZSubscriptionIdArg)
        from azure.cli.core.commands.parameters import get_location_completion_list
        from azure.cli.core._completers import get_subscription_id_list

        schema = AAZArgumentsSchema()
        v = schema()

        schema.foo = AAZResourceLocationArg(
            options=['foo'],
            completer=self.custom_completer
        )
        schema.bar = AAZResourceLocationArg(
            options=['bar']
        )
        schema.sub = AAZSubscriptionIdArg(
            options=["--subscription-id"],
            help="subscription id",
            required=True,
            id_part="subscription")
        self.assertEqual(schema.foo._completer, self.custom_completer)
        self.assertEqual(schema.bar._completer, get_location_completion_list)
        self.assertEqual(schema.sub._completer, get_subscription_id_list)

        args = {}
        for name, field in schema._fields.items():
            # generate command arguments from argument schema.
            try:
                args[name] = field.to_cmd_arg(name)
            except AAZUnregisteredArg:
                continue
        self.assertEqual(args['foo'].completer, self.custom_completer)
        self.assertEqual(args['bar'].completer, get_location_completion_list)
        self.assertEqual(args['sub'].completer, get_subscription_id_list)
        schema.bar._completer = self.custom_completer
        new_arg = schema.bar.to_cmd_arg('bar')
        self.assertEqual(new_arg.completer, self.custom_completer)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

from azure.core.exceptions import ResourceNotFoundError

from ._utils import resolve_store_metadata


def list_nsp_configurations(cmd, client, store_name, resource_group_name=None):
    if resource_group_name is None:
        resource_group_name, _ = resolve_store_metadata(cmd, store_name)
    return client.list_by_configuration_store(resource_group_name=resource_group_name, config_store_name=store_name)


def show_nsp_configuration(cmd, client, store_name, name, resource_group_name=None):
    if resource_group_name is None:
        resource_group_name, _ = resolve_store_metadata(cmd, store_name)
    try:
        return client.get(
            resource_group_name=resource_group_name,
            config_store_name=store_name,
            network_security_perimeter_configuration_name=name
        )
    except ResourceNotFoundError:
        raise ResourceNotFoundError(
            "The network security perimeter configuration '{}' for App Configuration '{}' was not found.".format(
                name, store_name))


def reconcile_nsp_configuration(cmd, client, store_name, name, resource_group_name=None):
    if resource_group_name is None:
        resource_group_name, _ = resolve_store_metadata(cmd, store_name)
    try:
        return client.begin_reconcile(
            resource_group_name=resource_group_name,
            config_store_name=store_name,
            network_security_perimeter_configuration_name=name
        )
    except ResourceNotFoundError:
        raise ResourceNotFoundError(
            "The network security perimeter configuration '{}' for App Configuration '{}' was not found.".format(
                name, store_name))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import collections

from azure.mgmt.core.tools import parse_resource_id


def aro_list_table_format(results):
    return [aro_show_table_format(r) for r in results]


def aro_show_table_format(result):
    parts = parse_resource_id(result['id'])

    worker_profiles = result['workerProfiles'] or []

    return collections.OrderedDict(
        Name=result['name'],
        ResourceGroup=parts['resource_group'],
        Location=result['location'],
        ProvisioningState=result['provisioningState'],
        WorkerCount=sum(wp['count'] or 0 for wp in worker_profiles),
        URL=result['consoleProfile']['url'],
    )


def aro_version_table_format(result):
    return [aro_build_version_table(r) for r in result]


def aro_build_version_table(result):
    return collections.OrderedDict(
        Version=result
    )



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
import tempfile
import unittest
import yaml
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, live_only, record_only


class AzureContainerInstanceScenarioTest(ScenarioTest):
    # Test create container with image, os type, ip address type, port, dns name label, cpu,
    # memory, command line and environment variables specified.
    @ResourceGroupPreparer()
    def test_container_create(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        dns_name_label = container_group_name
        fqdn = '{}.{}.azurecontainer.io'.format(
            container_group_name, resource_group_location)
        port1 = 8000
        port2 = 8001
        ports = '{} {}'.format(port1, port2)
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        restart_policy = 'Never'
        secrets = 'secret1=superawesomesecret secret2="nothing to see"'
        secret_path = '/s'
        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'dns_name_label': dns_name_label,
            'fqdn': fqdn,
            'ip_address_type': ip_address_type,
            'port1': port1,
            'port2': port2,
            'ports': ports,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'env': env,
            'restart_policy': restart_policy,
            'secrets': secrets,
            'secrets_mount_path': secret_path
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --dns-name-label {dns_name_label} --ports {ports} --cpu {cpu} --memory {memory} '
                 '--command-line {command} -e {env} --restart-policy {restart_policy} '
                 '--secrets {secrets} --secrets-mount-path {secrets_mount_path}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('ipAddress.dnsNameLabel',
                                    '{container_group_name}'),
                         self.check('ipAddress.fqdn', '{fqdn}'),
                         self.exists('ipAddress.ip'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('volumes'),
                         self.check('volumes[0].secret', {})])

        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.exists('ipAddress.ip'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check('containers[0].resources.requests.memoryInGb', memory)])

        # Test list
        self.cmd('container list -g {rg}',
                 checks=[self.check('[0].name', '{container_group_name}'),
                         self.check('[0].location',
                                    '{resource_group_location}'),
                         self.check('[0].provisioningState', 'Succeeded'),
                         self.check('[0].osType', '{os_type}'),
                         self.check('[0].restartPolicy', '{restart_policy}'),
                         self.exists('[0].ipAddress.ip'),
                         self.exists('[0].ipAddress.ports'),
                         self.check('[0].ipAddress.ports[0].port', '{port1}'),
                         self.check('[0].ipAddress.ports[1].port', '{port2}'),
                         self.check('[0].containers[0].image', '{image}'),
                         self.exists('[0].containers[0].command'),
                         self.exists('[0].containers[0].environmentVariables'),
                         self.check(
                             '[0].containers[0].resources.requests.cpu', cpu),
                         self.check('[0].containers[0].resources.requests.memoryInGb', memory)])

        # Test logs
        self.cmd('container logs -g {rg} -n {container_group_name}')

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('ipAddress.dnsNameLabel',
                                    '{container_group_name}'),
                         self.check('ipAddress.fqdn', '{fqdn}'),
                         self.exists('ipAddress.ip'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('volumes'),
                         self.check('volumes[0].secret', {})])
						 
    # Test create container with Spot Priority
    @ResourceGroupPreparer()
    def test_container_create_spot_priority(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        restart_policy = 'Never'
        priority = 'Spot'
        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type, 
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'env': env,
            'restart_policy': restart_policy,
            'priority' : priority
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--cpu {cpu} --memory {memory} --priority {priority} '
                 '--command-line {command} -e {env} --restart-policy {restart_policy}  ',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test logs
        self.cmd('container logs -g {rg} -n {container_group_name}')

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

    # Test create container with Regular Priority
    @ResourceGroupPreparer()
    def test_container_create_regular_priority(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        restart_policy = 'Never'
        priority = 'Regular'
        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type, 
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'env': env,
            'restart_policy': restart_policy,
            'priority' : priority
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--cpu {cpu} --memory {memory} --priority {priority} '
                 '--command-line {command} -e {env} --restart-policy {restart_policy}  ',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test logs
        self.cmd('container logs -g {rg} -n {container_group_name}')

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

    # Test create container using managed identities.
    @ResourceGroupPreparer()
    def test_container_create_with_msi(self, resource_group, resource_group_location):
        container_group_name1 = self.create_random_name('clicontainer', 16)
        container_group_name2 = self.create_random_name('clicontainer', 16)
        container_group_name3 = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        user_assigned_identity_name = self.create_random_name('cliaciidentity', 20)
        system_assigned_identity = '[system]'
        cpu = 1
        memory = 1

        self.kwargs.update({
            'user_assigned_identity_name': user_assigned_identity_name,
        })

        msi_identity_result = self.cmd('identity create -g {rg} -n {user_assigned_identity_name}').get_output_in_json()

        self.kwargs.update({
            'container_group_name1': container_group_name1,
            'container_group_name2': container_group_name2,
            'container_group_name3': container_group_name3,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'user_assigned_identity': msi_identity_result['id'],
            'system_assigned_identity': system_assigned_identity,
            'cpu': 1,
            'memory': 1
        })

        # Test create system assigned identity
        self.cmd('container create -g {rg} -n {container_group_name1} --image {image} --os-type {os_type} --cpu {cpu} --memory {memory} '
                 '--ip-address {ip_address_type} --assign-identity',
                 checks=[self.check('name', '{container_group_name1}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('identity.type', 'SystemAssigned'),
                         self.exists('ipAddress.ip'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}')])

        # Test create user assigned identity
        self.cmd('container create -g {rg} -n {container_group_name2} --image {image} --os-type {os_type} --cpu {cpu} --memory {memory} '
                 '--ip-address {ip_address_type} --assign-identity {user_assigned_identity}',
                 checks=[self.check('name', '{container_group_name2}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('identity.type', 'UserAssigned'),
                         self.exists('ipAddress.ip'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}')])

        # Test create system user assigned identity
        self.cmd('container create -g {rg} -n {container_group_name3} --image {image} --os-type {os_type} --cpu {cpu} --memory {memory} '
                 '--ip-address {ip_address_type} --assign-identity {system_assigned_identity} {user_assigned_identity}',
                 checks=[self.check('name', '{container_group_name3}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('identity.type', 'SystemAssigned, UserAssigned'),
                         self.exists('ipAddress.ip'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}')])

    # Test create container using managed identities with scope.
    @live_only()
    @ResourceGroupPreparer()
    def test_container_create_with_msi_scope(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        cpu = 1
        memory = 1
        storage_account_name = self.create_random_name('clistorage', 16)

        self.kwargs.update({
            'storage_account_name': storage_account_name
        })

        storage_account_result = self.cmd('az storage account create -n {storage_account_name} -g {rg} ').get_output_in_json()

        self.kwargs.update({
            'container_group_name1': container_group_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'cpu': cpu,
            'memory': memory,
            'msi_scope': storage_account_result['id']
        })

        # Test create system assigned identity with scope
        self.cmd('container create -g {rg} -n {container_group_name1} --image {image} --os-type {os_type} --cpu {cpu} --memory {memory} '
                 '--ip-address {ip_address_type} --assign-identity --scope {msi_scope}',
                 checks=[self.check('name', '{container_group_name1}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('identity.type', 'SystemAssigned'),
                         self.exists('ipAddress.ip'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}')])

    # Test create container with azure container registry image.
    # An ACR instance is required to re-record this test with 'nginx:latest' image available in the url.
    # see https://learn.microsoft.com/azure/container-registry/container-registry-get-started-docker-cli
    # After recording, regenerate the password for the acr instance.
    @record_only()  # This test relies on existing ACR image
    @ResourceGroupPreparer()
    def test_container_create_with_acr(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        registry_username = 'azclitest'
        registry_server = '{}.azurecr.io'.format(registry_username)
        image = '{}/nginx:latest'.format(registry_server)
        password = 'passplaceholder'
        cpu = 1
        memory = 1
        os_type = 'Linux'

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'registry_username': registry_username,
            'registry_server': registry_server,
            'image': image,
            'password': password,
            'cpu': cpu,
            'memory': memory,
            'os_type': os_type
        })

        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --registry-username {registry_username} --registry-password {password} --cpu {cpu} --memory {memory} --os-type {os_type}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.check(
                             'imageRegistryCredentials[0].server', '{registry_server}'),
                         self.check(
                             'imageRegistryCredentials[0].username', '{registry_username}'),
                         self.exists('containers[0].resources.requests.cpu'),
                         self.exists('containers[0].resources.requests.memoryInGb')])

        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.check(
                             'imageRegistryCredentials[0].server', '{registry_server}'),
                         self.check(
                             'imageRegistryCredentials[0].username', '{registry_username}'),
                         self.exists('containers[0].resources.requests.cpu'),
                         self.exists('containers[0].resources.requests.memoryInGb')])

    # Test create container with VNET argument validations.
    @ResourceGroupPreparer()
    def test_container_create_with_vnet(self, resource_group, resource_group_location):
        from azure.core.exceptions import HttpResponseError
        from knack.util import CLIError

        test_sub_id = '00000000-0000-0000-0000-000000000000'
        container_group_name = self.create_random_name('clicontainer', 16)
        vnet_name = self.create_random_name('vent', 16)
        subnet_name = self.create_random_name('subnet', 16)
        ip_address_type = "Private"

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'vnet_name': vnet_name,
            'subnet_name': subnet_name,
            'ip_addresss': ip_address_type
        })

        # Create NSG
        self.cmd('network nsg create -g {rg} -n {vnet_name}nsg')

        # Create VNET
        self.cmd('network vnet create -g {rg} -n {vnet_name} --subnet-name {subnet_name} --address-prefixes 10.0.0.0/24 --network-security-group {vnet_name}nsg')

        # Vnet name with no subnet
        with self.assertRaisesRegex(CLIError, "usage error: --vnet NAME --subnet NAME | --vnet ID --subnet NAME | --subnet ID"):
            self.cmd('container create -g {rg} -n {container_group_name} --image mcr.microsoft.com/azuredocs/aci-helloworld:latest --vnet {vnet_name} --cpu 1 --memory 1 --os-type Linux')

        # Subnet name with no vnet name
        with self.assertRaisesRegex(CLIError, "usage error: --vnet NAME --subnet NAME | --vnet ID --subnet NAME | --subnet ID"):
            self.cmd('container create -g {rg} -n {container_group_name} --image mcr.microsoft.com/azuredocs/aci-helloworld:latest --cpu 1 --memory 1 --os-type Linux '
                     '--subnet {subnet_name} ')

        self.cmd('container create -g {rg} -n {container_group_name} --image mcr.microsoft.com/azuredocs/aci-helloworld:latest --vnet {vnet_name} --subnet {subnet_name} --ip-address {ip_addresss} --cpu 1 --memory 1 --os-type Linux',
            checks=[self.exists('subnetIds[0].id')])


    # Test export container.
    @ResourceGroupPreparer()
    def test_container_export(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        cpu = 1
        memory = 1

        _, output_file = tempfile.mkstemp()

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'output_file': output_file,
            'image': image,
            'cpu': cpu,
            'memory': memory,
            'os_type': os_type
        })

        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --cpu {cpu} --memory {memory} --os-type {os_type}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', 'Linux'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].resources.requests.cpu'),
                         self.exists('containers[0].resources.requests.memoryInGb')])
        self.cmd('container export -g {rg} -n {container_group_name} -f "{output_file}"')

        cg_definition = None
        with open(output_file, 'r') as f:
            cg_definition = yaml.safe_load(f)

        self.check(cg_definition["name"], container_group_name)
        self.check(cg_definition['properties']['containers'][0]['properties']['image'], image)
        self.check(cg_definition['location'], resource_group_location)
        self.check(cg_definition['properties']['containers'][0]['properties']['resources']['requests']['cpu'], 1.0)
        self.check(cg_definition['properties']['containers'][0]['properties']['resources']['requests']['memoryInGB'], 1.5)

    # Test export container with identity.
    @ResourceGroupPreparer()
    def test_container_export_with_identity(self, resource_group, resource_group_location):
        container_group_name1 = self.create_random_name('clicontainer', 16)
        container_group_name2 = self.create_random_name('clicontainer', 16)
        container_group_name3 = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        user_assigned_identity_name = self.create_random_name('cliaciidentity', 20)
        system_assigned_identity = '[system]'
        os_type = 'Linux'
        cpu = 1
        memory = 1

        self.kwargs.update({
            'user_assigned_identity_name': user_assigned_identity_name,
        })

        msi_identity_result = self.cmd('identity create -g {rg} -n {user_assigned_identity_name}').get_output_in_json()

        _, output_file = tempfile.mkstemp()

        def check_export_with_identity(container_group_name, identity_type):
            cg_definition = None
            with open(output_file, 'r') as f:
                cg_definition = yaml.safe_load(f)

            self.check(cg_definition["name"], container_group_name)
            self.check(cg_definition['properties']['containers'][0]['properties']['image'], image)
            self.check(cg_definition['location'], resource_group_location)
            self.check(cg_definition['identity']['type'], identity_type)
            if 'UserAssigned' in identity_type:
                self.exists(cg_definition['identity']['user_assigned_identities'])

        self.kwargs.update({
            'container_group_name1': container_group_name1,
            'container_group_name2': container_group_name2,
            'container_group_name3': container_group_name3,
            'resource_group_location': resource_group_location,
            'output_file': output_file,
            'image': image,
            'cpu': cpu,
            'memory': memory,
            'os_type': os_type,
            'user_assigned_identity': msi_identity_result['id'],
            'system_assigned_identity': system_assigned_identity,
        })

        # Test create system assigned identity
        self.cmd('container create -g {rg} -n {container_group_name1} --image {image} --cpu {cpu} --memory {memory} --os-type {os_type} --assign-identity',
                 checks=[self.check('name', '{container_group_name1}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}'),
                         self.check('identity.type', 'SystemAssigned')])
        self.cmd('container export -g {rg} -n {container_group_name1} -f "{output_file}"')
        check_export_with_identity(container_group_name1, 'SystemAssigned')

        # Test create user assigned identity
        self.cmd('container create -g {rg} -n {container_group_name2} --image {image} --cpu {cpu} --memory {memory} --os-type {os_type} --assign-identity {user_assigned_identity}',
                 checks=[self.check('name', '{container_group_name2}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}'),
                         self.check('identity.type', 'UserAssigned')])
        self.cmd('container export -g {rg} -n {container_group_name2} -f "{output_file}"')
        check_export_with_identity(container_group_name2, 'UserAssigned')

        # Test create system user assigned identity
        self.cmd('container create -g {rg} -n {container_group_name3} --image {image} --cpu {cpu} --memory {memory} --os-type {os_type} --assign-identity {system_assigned_identity} {user_assigned_identity}',
                 checks=[self.check('name', '{container_group_name3}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('containers[0].image', '{image}'),
                         self.check('identity.type', 'SystemAssigned, UserAssigned')])
        self.cmd('container export -g {rg} -n {container_group_name3} -f "{output_file}"')
        check_export_with_identity(container_group_name3, 'SystemAssigned, UserAssigned')

    # Test create container with azure file volume
    @ResourceGroupPreparer()
    @unittest.skip("Skip test as unable to re-record due to missing pre-req. resources.")
    def test_container_azure_file_volume_mount(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        azure_file_volume_share_name = 'testshare'
        azure_file_volume_account_name = 'ccondemostore1'
        azure_file_volume_account_key = 'mockstorageaccountkey'
        azure_file_volume_mount_path = '/mnt/azfile'

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'azure_file_volume_share_name': azure_file_volume_share_name,
            'azure_file_volume_account_name': azure_file_volume_account_name,
            'azure_file_volume_account_key': azure_file_volume_account_key,
            'azure_file_volume_mount_path': azure_file_volume_mount_path,
        })

        self.cmd('container create -g {rg} -n {container_group_name} --image nginx '
                 '--azure-file-volume-share-name {azure_file_volume_share_name} '
                 '--azure-file-volume-account-name {azure_file_volume_account_name} '
                 '--azure-file-volume-account-key {azure_file_volume_account_key} '
                 '--azure-file-volume-mount-path {azure_file_volume_mount_path}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', 'Linux'),
                         self.exists('volumes'),
                         self.exists('volumes[0].azureFile'),
                         self.check(
                             'volumes[0].azureFile.shareName', '{azure_file_volume_share_name}'),
                         self.check(
                             'volumes[0].azureFile.storageAccountName', '{azure_file_volume_account_name}'),
                         self.exists('containers[0].volumeMounts'),
                         self.check('containers[0].volumeMounts[0].mountPath', '{azure_file_volume_mount_path}')])

        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', 'Linux'),
                         self.exists('volumes'),
                         self.exists('volumes[0].azureFile'),
                         self.check(
                             'volumes[0].azureFile.shareName', '{azure_file_volume_share_name}'),
                         self.check(
                             'volumes[0].azureFile.storageAccountName', '{azure_file_volume_account_name}'),
                         self.exists('containers[0].volumeMounts'),
                         self.check('containers[0].volumeMounts[0].mountPath', '{azure_file_volume_mount_path}')])

    # Test create container with git repo volume
    @ResourceGroupPreparer()
    @unittest.skip("Skip test as unable to re-record due to missing pre-req. resources.")
    def test_container_git_repo_volume_mount(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        gitrepo_url = 'https://github.com/yolo3301/dumb-flow.git'
        gitrepo_dir = './test'
        gitrepo_revision = '5604f0a8f11bfe13e621418ab6f6a71973e208ce'
        gitrepo_mount_path = '/src'

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'gitrepo_url': gitrepo_url,
            'gitrepo_dir': gitrepo_dir,
            'gitrepo_revision': gitrepo_revision,
            'gitrepo_mount_path': gitrepo_mount_path,
        })

        self.cmd('container create -g {rg} -n {container_group_name} --image nginx '
                 '--gitrepo-url {gitrepo_url} '
                 '--gitrepo-dir {gitrepo_dir} '
                 '--gitrepo-revision {gitrepo_revision} '
                 '--gitrepo-mount-path {gitrepo_mount_path}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', 'Linux'),
                         self.exists('volumes'),
                         self.exists('volumes[0].gitRepo'),
                         self.check(
                             'volumes[0].gitRepo.repository', '{gitrepo_url}'),
                         self.check(
                             'volumes[0].gitRepo.directory', '{gitrepo_dir}'),
                         self.check(
                             'volumes[0].gitRepo.revision', '{gitrepo_revision}'),
                         self.exists('containers[0].volumeMounts'),
                         self.check('containers[0].volumeMounts[0].mountPath', '{gitrepo_mount_path}')])

        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', 'Linux'),
                         self.exists('volumes'),
                         self.exists('volumes[0].gitRepo'),
                         self.check(
                             'volumes[0].gitRepo.repository', '{gitrepo_url}'),
                         self.check(
                             'volumes[0].gitRepo.directory', '{gitrepo_dir}'),
                         self.check(
                             'volumes[0].gitRepo.revision', '{gitrepo_revision}'),
                         self.exists('containers[0].volumeMounts'),
                         self.check('containers[0].volumeMounts[0].mountPath', '{gitrepo_mount_path}')])

    # Changing to live only because of intermittent test failures: https://github.com/Azure/azure-cli/issues/19804
    @live_only()
    @ResourceGroupPreparer()
    def test_container_attach(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'echo hello; sleep 15; done\'"'
        restart_policy = 'Never'

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'restart_policy': restart_policy,
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --cpu {cpu} --memory {memory} '
                 '--command-line {command} --restart-policy {restart_policy} ',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        self.cmd('container attach -g {rg} -n {container_group_name}')

    # test is live only because repo test environment does not have a stdin file pointer
    # ie. "UnsupportedOperation("redirected stdin is pseudofile, has no fileno()")"
    @live_only()
    @ResourceGroupPreparer()
    def test_container_exec(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'echo hello; sleep 15; done\'"'
        restart_policy = 'Never'

        self.kwargs.update({
            'container_group_name': container_group_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'restart_policy': restart_policy,
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --cpu {cpu} --memory {memory} '
                 '--command-line {command} --restart-policy {restart_policy} ',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test exec
        self.cmd('container exec -g {rg} -n {container_group_name} --exec-command \"ls\"')


    # Test container create with a zone specified
    @ResourceGroupPreparer()
    def test_container_create_with_zone(self, resource_group, resource_group_location):
        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        restart_policy = 'Never'
        zone = '1'
        location = "eastus"

        self.kwargs.update({
            'container_group_name': container_group_name,
            'location': location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'restart_policy': restart_policy,
            'zone': zone
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --cpu {cpu} --memory {memory} --zone {zone} '
                 '--command-line {command} --restart-policy {restart_policy} --location {location}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('zones[0]', zone)])

    # Test container with Confidential SKU
    @ResourceGroupPreparer()
    def test_container_create_with_confidential_sku(self, resource_group, resource_group_location):

        container_group_name = self.create_random_name('clicontainer', 16)
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        restart_policy = 'Never'
        location = "northeurope"
        sku="Confidential"
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='

        self.kwargs.update({
            'container_group_name': container_group_name,
            'location': location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'restart_policy': restart_policy,
            'sku': sku,
            'env': env
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --cpu {cpu} --memory {memory} --sku {sku} '
                 '--command-line {command} --restart-policy {restart_policy} --location {location} -e {env} '
                 '--privileged --allow-escalation',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('containers[0].securityContext'),
                         self.check('containers[0].securityContext.privileged', True),
                         self.check('sku', sku),
                         self.exists('confidentialComputeProperties.ccePolicy')])
        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('containers[0].securityContext'),
                         self.check('containers[0].securityContext.privileged', True),
                         self.check('containers[0].securityContext.allowPrivilegeEscalation', True),
                         self.check('sku', sku),
                         self.exists('confidentialComputeProperties.ccePolicy')])

        # Test logs
        self.cmd('container logs -g {rg} -n {container_group_name}')

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.check('sku', sku),
                         self.exists('confidentialComputeProperties.ccePolicy')])

    # Test container reuse from standby pool with config maps
    # This test relies on existing container group profile and standby pool profile
    @ResourceGroupPreparer()
    def test_container_reuse_from_standbypool_outbound(self, resource_group, resource_group_location):

        container_group_name = self.create_random_name('clicontainer', 16)
        container_group_profile_id = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.ContainerInstance/containerGroupProfiles/testcgp-outbound'
        container_group_profile_revision = '1'
        standby_pool_profile_id = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.StandbyPool/standbyContainerGroupPools/testpool-outbound'
        location = "eastus"
        config_map = 'KEY1=VALUE1 KEY2=VALUE2'
        cpu = 1
        memory = 1
        os_type = 'Linux'
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        
        self.kwargs.update({
            'container_group_name': container_group_name,
            'location': location,
            'container_group_profile_id': container_group_profile_id,
            'container_group_profile_revision': container_group_profile_revision,
            'standby_pool_profile_id': standby_pool_profile_id,
            'config_map': config_map,
            'cpu': cpu,
            'memory': memory,
            'os_type': os_type,
            'image': image
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --config-map {config_map} '
                 '--container-group-profile-id {container_group_profile_id} --container-group-profile-revision {container_group_profile_revision} '
                 '--standby-pool-profile-id {standby_pool_profile_id} --location {location} ',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])
        
        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

    # Test container reuse from standby pool with config maps and public ip address
    # This test relies on existing container group profile and standby pool profile
    @ResourceGroupPreparer()
    def test_container_reuse_from_standbypool(self, resource_group, resource_group_location):

        container_group_name = self.create_random_name('clicontainer', 16)
        container_group_profile_id = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.ContainerInstance/containerGroupProfiles/testcg'
        container_group_profile_revision = '1'
        standby_pool_profile_id = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.StandbyPool/standbyContainerGroupPools/testpool'
        location = "eastus"
        config_map = 'KEY1=VALUE1 KEY2=VALUE2'
        cpu = 1
        memory = 1
        os_type = 'Linux'
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        
        self.kwargs.update({
            'container_group_name': container_group_name,
            'location': location,
            'container_group_profile_id': container_group_profile_id,
            'container_group_profile_revision': container_group_profile_revision,
            'standby_pool_profile_id': standby_pool_profile_id,
            'config_map': config_map,
            'cpu': cpu,
            'memory': memory,
            'os_type': os_type,
            'image': image
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --config-map {config_map} '
                 '--container-group-profile-id {container_group_profile_id} --container-group-profile-revision {container_group_profile_revision} '
                 '--standby-pool-profile-id {standby_pool_profile_id} --location {location} ',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check('ipAddress.type', 'Public'),
                         self.exists('ipAddress.ip'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])
        
        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

    # Test container reuse from standby pool with delegated subnet 
    # This test relies on existing container group profile, standby pool profile and virtual network
    @ResourceGroupPreparer()
    def test_container_reuse_from_standbypool_vnet(self, resource_group, resource_group_location):

        container_group_name = self.create_random_name('clicontainer', 16)
        container_group_profile_id = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.ContainerInstance/containerGroupProfiles/testcgpvnet'
        container_group_profile_revision = '1'
        standby_pool_profile_id = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.StandbyPool/standbyContainerGroupPools/testvnetpool'
        location = "eastus"
        config_map = 'KEY1=VALUE1 KEY2=VALUE2'
        vnet_name = '/subscriptions/da28f5e5-aa45-46fe-90c8-053ca49ab4b5/resourceGroups/azcliresources/providers/Microsoft.Network/virtualNetworks/testvnet'
        subnet_name = 'subnet1'
        cpu = 1
        memory = 1.5
        os_type = 'Linux'
        image = 'mcr.microsoft.com/azuredocs/aci-helloworld:latest'
        
        self.kwargs.update({
            'container_group_name': container_group_name,
            'location': location,
            'container_group_profile_id': container_group_profile_id,
            'container_group_profile_revision': container_group_profile_revision,
            'standby_pool_profile_id': standby_pool_profile_id,
            'config_map': config_map,
            'vnet_name': vnet_name,
            'subnet_name': subnet_name,
            'cpu': cpu,
            'memory': memory,
            'os_type': os_type,
            'image': image
        })

        # Test create
        self.cmd('container create -g {rg} -n {container_group_name} --config-map {config_map} '
                 '--container-group-profile-id {container_group_profile_id} --container-group-profile-revision {container_group_profile_revision} '
                 '--standby-pool-profile-id {standby_pool_profile_id} --vnet {vnet_name} --subnet {subnet_name} --location {location}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check('ipAddress.type', 'Private'),
                         self.exists('ipAddress.ip'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)]),

        
        # Test show
        self.cmd('container show -g {rg} -n {container_group_name}',
                 checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test delete
        self.cmd('container delete -g {rg} -n {container_group_name} -y',
            checks=[self.check('name', '{container_group_name}'),
                         self.check('location', '{location}'),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('isCreatedFromStandbyPool','True'),
                         self.check('osType', '{os_type}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].configMap'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

    # Test create container container-group-profile with image, os type, ip address type, port, cpu,
    # memory, command line and environment variables specified.
    @ResourceGroupPreparer()
    def test_container_group_profile_create(self, resource_group, resource_group_location):
        container_group_profile_name = self.create_random_name('clicgp', 16)
        image = 'alpine:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        port1 = 8000
        port2 = 8001
        ports = '{} {}'.format(port1, port2)
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        restart_policy = 'Never'
        secrets = 'secret1=superawesomesecret secret2="nothing to see"'
        secret_path = '/s'
        self.kwargs.update({
            'container_group_profile_name': container_group_profile_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'port1': port1,
            'port2': port2,
            'ports': ports,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'env': env,
            'restart_policy': restart_policy,
            'secrets': secrets,
            'secrets_mount_path': secret_path
        })

        # Test create
        self.cmd('container container-group-profile create -g {rg} -n {container_group_profile_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --ports {ports} --cpu {cpu} --memory {memory} '
                 '--command-line {command} -e {env} --restart-policy {restart_policy} '
                 '--secrets {secrets} --secrets-mount-path {secrets_mount_path}',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('volumes'),
                         self.check('volumes[0].secret', {})])

        # Test show
        self.cmd('container container-group-profile show -g {rg} -n {container_group_profile_name}',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check('containers[0].resources.requests.memoryInGb', memory)])
        
        # Test show by revision number 
        self.cmd('container container-group-profile show-revision -g {rg} -n {container_group_profile_name} -r 1',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check('containers[0].resources.requests.memoryInGb', memory)])
        
        # Test delete
        self.cmd('container container-group-profile delete -g {rg} -n {container_group_profile_name} -y')
        
    # Test create container container-group-profile with Confidential SKU
    @ResourceGroupPreparer()
    def test_container_group_profile_create_with_confidential_sku(self, resource_group, resource_group_location):

        container_group_profile_name = self.create_random_name('clicgp', 16)
        image = 'alpine:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        cpu = 1
        memory = 1
        restart_policy = 'Never'
        location = "northeurope"
        sku="Confidential"
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        cce_policy = "eyJhbGxvd19hbGwiOiB0cnVlLCAiY29udGFpbmVycyI6IHsibGVuZ3RoIjogMCwgImVsZW1lbnRzIjogbnVsbH19"

        self.kwargs.update({
            'container_group_profile_name': container_group_profile_name,
            'location': location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'cpu': cpu,
            'memory': memory,
            'restart_policy': restart_policy,
            'sku': sku,
            'env': env,
            'cce_policy': cce_policy
        })

        # Test create
        self.cmd('container container-group-profile create -g {rg} -n {container_group_profile_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --cpu {cpu} --memory {memory} --sku {sku} '
                 '--restart-policy {restart_policy} --location {location} -e {env} --cce-policy {cce_policy} '
                 '--privileged --allow-escalation',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('containers[0].securityContext'),
                         self.check('containers[0].securityContext.privileged', True),
                         self.check('sku', sku),
                         self.exists('confidentialComputeProperties.ccePolicy')])
        
        # Test show
        self.cmd('container container-group-profile show -g {rg} -n {container_group_profile_name}',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory),
                         self.exists('containers[0].securityContext'),
                         self.check('containers[0].securityContext.privileged', True),
                         self.check('containers[0].securityContext.allowPrivilegeEscalation', True),
                         self.check('sku', sku),
                         self.exists('confidentialComputeProperties.ccePolicy')])

        # Test delete
        self.cmd('container container-group-profile delete -g {rg} -n {container_group_profile_name} -y')
        

    # Test create container container-group-profile with Spot Priority
    @ResourceGroupPreparer()
    def test_container_group_profile_create_spot_priority(self, resource_group, resource_group_location):
        container_group_profile_name = self.create_random_name('clicgp', 16)
        image = 'alpine:latest'
        os_type = 'Linux'
        cpu = 1
        memory = 1
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        restart_policy = 'Never'
        priority = 'Spot'
        self.kwargs.update({
            'container_group_profile_name': container_group_profile_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type, 
            'cpu': cpu,
            'memory': memory,
            'env': env,
            'restart_policy': restart_policy,
            'priority' : priority
        })

        # Test create
        self.cmd('container container-group-profile create -g {rg} -n {container_group_profile_name} --image {image} --os-type {os_type} '
                 '--cpu {cpu} --memory {memory} --priority {priority} '
                 '-e {env} --restart-policy {restart_policy}  ',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test show
        self.cmd('container container-group-profile show -g {rg} -n {container_group_profile_name}',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy}'),
                         self.check('priority', '{priority}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].environmentVariables'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])

        # Test delete
        self.cmd('container container-group-profile delete -g {rg} -n {container_group_profile_name} -y')

    @ResourceGroupPreparer()
    # Test container container-group-profile create with varied restart policies
    def test_container_group_profile_update(self, resource_group, resource_group_location):
        container_group_profile_name = self.create_random_name('clicgp', 16)
        image = 'alpine:latest'
        os_type = 'Linux'
        ip_address_type = 'Public'
        port1 = 8000
        port2 = 8001
        ports = '{} {}'.format(port1, port2)
        cpu = 1
        memory = 1
        command = '"/bin/sh -c \'while true; do echo hello; sleep 20; done\'"'
        env = 'KEY1=VALUE1 KEY2=FOO=BAR='
        restart_policy1 = 'Never'
        restart_policy2 = 'OnFailure'
        self.kwargs.update({
            'container_group_profile_name': container_group_profile_name,
            'resource_group_location': resource_group_location,
            'image': image,
            'os_type': os_type,
            'ip_address_type': ip_address_type,
            'port1': port1,
            'port2': port2,
            'ports': ports,
            'cpu': cpu,
            'memory': memory,
            'command': command,
            'env': env,
            'restart_policy1': restart_policy1,
            'restart_policy2': restart_policy2
        })

        # Test create with restart policy Never
        self.cmd('container container-group-profile create -g {rg} -n {container_group_profile_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --ports {ports} --cpu {cpu} --memory {memory} '
                 '--command-line {command} -e {env} --restart-policy {restart_policy1} ',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy1}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])
        
        # Test create with restart policy OnFailure
        self.cmd('container container-group-profile create -g {rg} -n {container_group_profile_name} --image {image} --os-type {os_type} '
                 '--ip-address {ip_address_type} --ports {ports} --cpu {cpu} --memory {memory} '
                 '--command-line {command} -e {env} --restart-policy {restart_policy2} ',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '2'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy2}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check(
                             'containers[0].resources.requests.memoryInGb', memory)])
    
        # Test show by revision number 
        self.cmd('container container-group-profile show-revision -g {rg} -n {container_group_profile_name} -r 1',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '1'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy1}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check('containers[0].resources.requests.memoryInGb', memory)])
        
        self.cmd('container container-group-profile show-revision -g {rg} -n {container_group_profile_name} -r 2',
                 checks=[self.check('name', '{container_group_profile_name}'),
                         self.check('location', '{resource_group_location}'),
                         self.check('revision', '2'),
                         self.check('osType', '{os_type}'),
                         self.check('restartPolicy', '{restart_policy2}'),
                         self.exists('ipAddress.ports'),
                         self.check('ipAddress.ports[0].port', '{port1}'),
                         self.check('ipAddress.ports[1].port', '{port2}'),
                         self.check('containers[0].image', '{image}'),
                         self.exists('containers[0].command'),
                         self.check(
                             'containers[0].resources.requests.cpu', cpu),
                         self.check('containers[0].resources.requests.memoryInGb', memory)])
        
        # Test delete
        self.cmd('container container-group-profile delete -g {rg} -n {container_group_profile_name} -y')

    
        
    


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import sys
import subprocess

python_version = os.listdir('/usr/lib64/az/lib/')[0]
root_dir = f'/usr/lib64/az/lib/{python_version}/site-packages/azure/cli/command_modules'
mod_list = [mod for mod in sorted(os.listdir(root_dir)) if os.path.isdir(os.path.join(root_dir, mod)) and mod != '__pycache__']

pytest_base_cmd = f'PYTHONPATH=/usr/lib64/az/lib/{python_version}/site-packages python -m pytest -v --forked -p no:warnings --log-level=WARN'
pytest_parallel_cmd = '{} -n logical'.format(pytest_base_cmd)

# cloud: https://github.com/Azure/azure-cli/pull/14994
# appservice: https://github.com/Azure/azure-cli/pull/19810
# iot, resource, azure-cli-core: https://github.com/Azure/azure-cli/pull/26176
serial_test_modules = ['botservice', 'network', 'cloud', 'appservice', 'iot', 'resource']

for mod_name in mod_list:
    cmd = '{} --junit-xml /azure_cli_test_result/{}.xml --pyargs azure.cli.command_modules.{}'.format(
        pytest_base_cmd if mod_name in serial_test_modules else pytest_parallel_cmd, mod_name, mod_name)
    print('Running:', cmd, flush=True)
    exit_code = subprocess.call(cmd, shell=True)
    if exit_code == 5:
        print('No tests found for {}'.format(mod_name))
    elif exit_code != 0:
        sys.exit(exit_code)

exit_code = subprocess.call(['{} --junit-xml /azure_cli_test_result/azure-cli-core.xml --pyargs azure.cli.core'.format(pytest_base_cmd)], shell=True)
sys.exit(exit_code)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.decorators import Completer
from azure.cli.core._profile import Profile


def get_keyvault_name_completion_list(resource_name):

    @Completer
    def completer(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
        func_name = 'list_properties_of_{}s'.format(resource_name)
        vault = namespace.vault_base_url
        profile = Profile(cli_ctx=cmd.cli_ctx)
        credential, _, _ = profile.get_login_credentials(subscription_id=cmd.cli_ctx.data.get('subscription_id'))
        if resource_name == 'key':
            from azure.keyvault.keys import KeyClient
            client = KeyClient(vault_url=vault, credential=credential, api_version='7.6-preview.2',
                               verify_challenge_resource=False)
        elif resource_name == 'secret':
            from azure.keyvault.secrets import SecretClient
            client = SecretClient(vault_url=vault, credential=credential, api_version='7.4',
                                  verify_challenge_resource=False)
        else:
            from azure.keyvault.certificates import CertificateClient
            client = CertificateClient(vault_url=vault, credential=credential, api_version='7.4',
                                       verify_challenge_resource=False)
        items = []
        for y in list(getattr(client, func_name)()):
            items.append(y.name)
        return items

    return completer


def get_keyvault_version_completion_list(resource_name):

    @Completer
    def completer(cmd, prefix, namespace, **kwargs):  # pylint: disable=unused-argument
        func_name = 'list_properties_of_{}_versions'.format(resource_name)
        vault = namespace.vault_base_url
        profile = Profile(cli_ctx=cmd.cli_ctx)
        credential, _, _ = profile.get_login_credentials(subscription_id=cmd.cli_ctx.data.get('subscription_id'))
        if resource_name == 'key':
            from azure.keyvault.keys import KeyClient
            client = KeyClient(vault_url=vault, credential=credential, api_version='7.6-preview.2',
                               verify_challenge_resource=False)
        elif resource_name == 'secret':
            from azure.keyvault.secrets import SecretClient
            client = SecretClient(vault_url=vault, credential=credential, api_version='7.4',
                                  verify_challenge_resource=False)
        else:
            from azure.keyvault.certificates import CertificateClient
            client = CertificateClient(vault_url=vault, credential=credential, api_version='7.4',
                                       verify_challenge_resource=False)
        items = []
        for y in list(getattr(client, func_name)()):
            items.append(y.version)
        return items

    return completer
